(function(root, factory) {
  if (typeof define === 'function' && define.amd) {
    // AMD. Register as an anonymous module.
    define(['ApiClient'], factory);
  } else if (typeof module === 'object' && module.exports) {
    // CommonJS-like environments that support module.exports, like Node.
    module.exports = factory(require('../ApiClient'));
  } else {
    // Browser globals (root is window)
    if (!root.OrbipayPaymentsapiClient) {
      root.OrbipayPaymentsapiClient = {};
    }
    root.OrbipayPaymentsapiClient.FeeResponse = factory(root.OrbipayPaymentsapiClient.ApiClient);
  }
}(this, function(ApiClient) {
  'use strict';




  /**
   * The FeeResponse model module.
   * @module models/FeeResponse
   */

  /**
   * <h3 style="color:red"> This class subject to change without prior notice, Please dont use this class directly. </h3>

   * Constructs a new <code>FeeResponse</code>.
   * @alias module:models/FeeResponse
   * @class
   */
  var exports = function() {
    var _this = this;





  };

  /**
   * Constructs a <code>FeeResponse</code> from a plain JavaScript object, optionally creating a new instance.
   * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
   * @param {Object} data The plain JavaScript object bearing properties of interest.
   * @param {module:models/FeeResponse} obj Optional instance to populate.
   * @return {module:models/FeeResponse} The populated <code>FeeResponse</code> instance.
   */
  exports.constructFromObject = function(data, obj) {
    if (data) {
      obj = obj || new exports();

      if (data.hasOwnProperty('fee_type')) {
        obj['fee_type'] = ApiClient.convertToType(data['fee_type'], 'String');
      }
      if (data.hasOwnProperty('fee_amount')) {
        obj['fee_amount'] = ApiClient.convertToType(data['fee_amount'], 'String');
      }
      if (data.hasOwnProperty('id')) {
        obj['id'] = ApiClient.convertToType(data['id'], 'String');
      }
      if (data.hasOwnProperty('url')) {
        obj['url'] = ApiClient.convertToType(data['url'], 'String');
      }
    }
    return obj;
  };

  /**
   * The way the fee is being charged to the customer. Fee could be added to the amount being paid, or, it could be charged separately by the biller. The fee_type would default to add_to_principal if the fee is not configured for the client in EBPP.
   * @member {module:models/FeeResponse.FeeTypeEnum} fee_type
   */
  exports.prototype['fee_type'] = undefined;
  /**
   * The fee amount upto 2 decimal places.
   * @member {String} fee_amount
   */
  exports.prototype['fee_amount'] = undefined;
  /**
   * The unique identifier assigned by EBPP to the fee. In case of split fee.
   * @member {String} id
   */
  exports.prototype['id'] = undefined;
  /**
   * This URL fetches the details of fee as a payment resource. In case of split fee.
   * @member {String} url
   */
  exports.prototype['url'] = undefined;


  /**
   * Allowed values for the <code>fee_type</code> property.
   * @enum {String}
   * @readonly
   */
  exports.FeeTypeEnum = {
    /**
     * value: "add_to_principal"
     * @const
     */
    "add_to_principal": "add_to_principal",
    /**
     * value: "bill_separately"
     * @const
     */
    "bill_separately": "bill_separately"  };


  return exports;
}));


